<?php
namespace Drop\Oms\Controller\Download;

use Drop\Oms\Model\Service\PdfDownloader;
use Drop\Oms\Helper\ConfigurationHelper;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\Response\Http\FileFactory;
use Magento\Framework\Controller\ResultFactory;
use \Magento\Framework\Controller\Result\RawFactory;
use Magento\Framework\Message\ManagerInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

class RmaProforma extends Action
{
    private $helper;
    private $orderRepository;
    private $downloader;
    private $fileFactory;
    private $resultRawFactory;
    protected $messageManager;
    protected $resultFactory;

    /**
     * @param Context $context
     * @param ConfigurationHelper $helper
     * @param OrderRepositoryInterface $orderRepository
     * @param PdfDownloader $downloader
     * @param FileFactory $fileFactory
     * @param RawFactory $resultRawFactory
     * @param ManagerInterface $messageManager
     * @param ResultFactory $resultFactory
     */
    public function __construct(
        Context                  $context,
        ConfigurationHelper      $helper,
        OrderRepositoryInterface $orderRepository,
        PdfDownloader            $downloader,
        FileFactory              $fileFactory,
        RawFactory               $resultRawFactory,
        ManagerInterface         $messageManager,
        ResultFactory            $resultFactory
    )
    {
        parent::__construct($context);
        $this->helper = $helper;
        $this->orderRepository = $orderRepository;
        $this->downloader = $downloader;
        $this->fileFactory = $fileFactory;
        $this->resultRawFactory = $resultRawFactory;
        $this->messageManager = $messageManager;
        $this->resultFactory = $resultFactory;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Raw|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        if (!$this->helper->isPdfEnabled()) {
            $this->messageManager->addErrorMessage('This feature is disabled');
            $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
            $resultRedirect->setUrl($this->_redirect->getRefererUrl());
            return $resultRedirect;
        }

        $orderId = $this->getRequest()->getParam('order_id');
        if (!$order = $this->orderRepository->get($orderId)) {
            $this->messageManager->addErrorMessage('Order not found');
            $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
            $resultRedirect->setUrl($this->_redirect->getRefererUrl());
            return $resultRedirect;
        }

        if (!$file = $this->downloader->getRmaProforma($order)) {
            $this->messageManager->addErrorMessage('File recovery error, contact support');
            $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
            $resultRedirect->setUrl($this->_redirect->getRefererUrl());
            return $resultRedirect;
        }

        try {
            $this->fileFactory->create(
                $file['name'],
                base64_decode($file['content']),
                DirectoryList::MEDIA, //basedir
                'application/octet-stream',
                '' // content length will be dynamically calculated
            );
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
            $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
            $resultRedirect->setUrl($this->_redirect->getRefererUrl());
            return $resultRedirect;
        }
        $resultRaw = $this->resultRawFactory->create();
        return $resultRaw;
    }
}
